@@ -1,5 +1,6 @@
 #define R_NO_REMAP
 
+#include <math.h>
 #include "gsl_nls.h"
 
 /* cleanup memory */
@@ -10,14 +11,25 @@ static void C_nls_cleanup(void *data)
     /* free memory */
     if (pars->w)
         gsl_multifit_nlinear_free(pars->w);
-
+    if (pars->q)
+        gsl_qrng_free(pars->q);
+    if (pars->wts)
+        gsl_vector_free(pars->wts);
+    if (pars->mx)
+        gsl_matrix_free(pars->mx);
+    if (pars->mp)
+        gsl_vector_free(pars->mp);
+    if (pars->mpopt)
+        gsl_vector_free(pars->mpopt);
+    if (pars->diag)
+        gsl_vector_free(pars->diag);
 }
 
 /* function call w/ cleanup */
 SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts, SEXP control_int, SEXP control_dbl)
 {
     /* function arguments */
-    pdata pars = {fn, y, jac, fvv, env, start, swts, control_int, control_dbl, NULL};
+    pdata pars = {fn, y, jac, fvv, env, start, swts, control_int, control_dbl, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 
     /* safe function call */
     SEXP ans = R_ExecWithCleanup(C_nls_internal, &pars, C_nls_cleanup, &pars);
@@ -34,12 +46,23 @@ SEXP C_nls_internal(void *data)
     pdata *pars = data;
 
     /* initialize parameters */
-    int nprotect = 1;
-    SEXP startvec = PROTECT(Rf_coerceVector(pars->start, REALSXP));
-    R_len_t p = Rf_length(startvec);
+    int nprotect = 0;
     R_len_t n = Rf_length(pars->y);
     R_len_t niter = INTEGER_ELT(pars->control_int, 0);
     int verbose = INTEGER_ELT(pars->control_int, 1);
+    Rboolean mstart = Rf_isMatrix(pars->start);
+    SEXP startvec = NULL;
+    R_len_t p;
+    if (!mstart)
+    {
+        startvec = PROTECT(Rf_coerceVector(pars->start, REALSXP));
+        p = Rf_length(startvec);
+        nprotect++;
+    }
+    else
+    {
+        p = Rf_ncols(pars->start);
+    }
 
     /* control parameters */
     gsl_multifit_nlinear_parameters fdf_params = gsl_multifit_nlinear_default_parameters();
@@ -105,7 +128,11 @@ SEXP C_nls_internal(void *data)
     /* initialize data */
     SEXP xpar = Rf_install("par");
     SEXP fcall = PROTECT(Rf_lang2(pars->fn, xpar));
-    SEXP parnames = PROTECT(Rf_getAttrib(pars->start, R_NamesSymbol));
+    SEXP parnames = NULL;
+    if (!mstart)
+        parnames = PROTECT(Rf_getAttrib(pars->start, R_NamesSymbol));
+    else
+        parnames = PROTECT(VECTOR_ELT(Rf_getAttrib(pars->start, R_DimNamesSymbol), 1));
     nprotect += 2;
 
     fdata params;
@@ -117,6 +144,10 @@ SEXP C_nls_internal(void *data)
     params.y = pars->y;
     params.rho = pars->env;
     params.start = pars->start;
+    if (!mstart)
+        params.warn = TRUE;
+    else
+        params.warn = FALSE;
 
     if (verbose)
     {
@@ -156,34 +187,104 @@ SEXP C_nls_internal(void *data)
         nprotect++;
     }
 
-    /* set nls optimization parameters */
-    double *start1 = (double *)S_alloc(p, sizeof(double));
-    for (R_len_t k = 0; k < p; k++)
-        start1[k] = REAL_ELT(startvec, k);
-    gsl_vector_view par = gsl_vector_view_array(start1, p);
+    /* use weights */
+    pars->wts = gsl_vector_alloc(n);
+    if (!Rf_isNull(pars->swts))
+        for (R_len_t i = 0; i < n; i++)
+            gsl_vector_set(pars->wts, i, REAL_ELT(pars->swts, i));
 
     /* initialize solver */
     const gsl_multifit_nlinear_type *T = gsl_multifit_nlinear_trust;
 
     /* allocate workspace with default parameters */
     pars->w = gsl_multifit_nlinear_alloc(T, &fdf_params, n, p);
+    pars->mpopt = gsl_vector_alloc(p);
 
-    /* initialize solver with starting point and weights */
+    /* multistart algorithm */
+    if (mstart)
+    {
+        /* allocate qrng workspace */
+        if (p < 41)
+            pars->q = gsl_qrng_alloc(gsl_qrng_sobol, p);
+        else
+            pars->q = gsl_qrng_alloc(gsl_qrng_halton, p);
+
+        /* multistart parameters */
+        mdata mpars;
+        mpars.n = INTEGER_ELT(pars->control_int, 6);
+        mpars.p = INTEGER_ELT(pars->control_int, 7);
+        mpars.q = INTEGER_ELT(pars->control_int, 8);
+        mpars.s = INTEGER_ELT(pars->control_int, 9);
+        mpars.niter = INTEGER_ELT(pars->control_int, 10);
+        mpars.max = INTEGER_ELT(pars->control_int, 11);
+        mpars.minsp = INTEGER_ELT(pars->control_int, 12);
+        mpars.r = REAL_ELT(pars->control_dbl, 8);
+        mpars.tol = REAL_ELT(pars->control_dbl, 9);
+        mpars.ntix = (int *)S_alloc(mpars.n, sizeof(int));
+        mpars.qmp = (double *)S_alloc(p, sizeof(double));
+        mpars.startptr = REAL(pars->start);
+        mpars.mssr_order = (int *)R_alloc(mpars.n, sizeof(int));
+        mpars.mstop = GSL_CONTINUE;
+        mpars.mstarts = 0;
+        mpars.nsp = 0;
+        mpars.nwsp = 0;
+        mpars.mssropt = (double)GSL_POSINF;
+
+        SEXP mssr = PROTECT(Rf_allocVector(REALSXP, mpars.n));
+        SEXP mpar = PROTECT(Rf_allocVector(REALSXP, p));
+        Rf_setAttrib(mpar, R_NamesSymbol, parnames);
+        params.start = mpar;
+        nprotect += 2;
+
+        /* placeholder matrices/vectors */
+        pars->mx = gsl_matrix_alloc(mpars.n, p);
+        pars->mp = gsl_vector_alloc(p);
+        pars->diag = gsl_vector_alloc(p);
+        /* initially focus sampling around center points */
+        for (R_len_t k = 0; k < p; k++)
+            gsl_vector_set(pars->diag, k, (mpars.startptr[2 * k + 1] - mpars.startptr[2 * k]) / 20.0);
+
+        /* multi-start global iterations */
+        do
+        {
+            gsl_multistart_driver(pars, &mpars, &fdf, mssr, xtol, ftol, gtol, verbose);
+
+            /* check stopping criterion */
+            mpars.mstarts += 1;
+            if (mpars.mstarts > mpars.max)
+                mpars.mstop = GSL_EMAXITER;
+            if (mpars.nsp >= mpars.minsp && mpars.nwsp >= mpars.r * mpars.nsp)
+                mpars.mstop = GSL_SUCCESS;
+
+        } while (mpars.mstop == GSL_CONTINUE);
+        if (verbose)
+        {
+            if (mpars.mstop == GSL_SUCCESS)
+                Rprintf("multi-start algorithm finished successfully (NSP = %d, NWSP = %d, # iterations = %d)\n", mpars.nsp, mpars.nwsp, mpars.mstarts);
+            if (mpars.mstop == GSL_EMAXITER)
+                Rprintf("multi-start algorithm reached max. number of global iterations (NSP = %d, NWSP = %d, # iterations = %d)\n", mpars.nsp, mpars.nwsp, mpars.mstarts);
+            Rprintf("*******************\n");
+        }
+        /* add noise in case of zero residuals */
+        if(mpars.mssropt <= 0)
+        {
+            gsl_vector_set(pars->mpopt, 0, gsl_vector_get(pars->mpopt, 0) + xtol);
+        }
+    }
+    else /* single-start */
+    {
+        for (R_len_t k = 0; k < p; k++)
+            gsl_vector_set(pars->mpopt, k, REAL_ELT(startvec, k));
+    }
+
+    /* (re-)initialize solver w/ optimal start parameters */
     if (!Rf_isNull(pars->swts))
-    {
-        double *swts1 = (double *)S_alloc(n, sizeof(double));
-        for (R_len_t i = 0; i < n; i++)
-            swts1[i] = REAL_ELT(pars->swts, i);
-        gsl_vector_view wts = gsl_vector_view_array(swts1, n);
-        gsl_multifit_nlinear_winit(&par.vector, &wts.vector, &fdf, pars->w);
-    }
+        gsl_multifit_nlinear_winit(pars->mpopt, pars->wts, &fdf, pars->w);
     else
-    {
-        gsl_multifit_nlinear_init(&par.vector, &fdf, pars->w);
-    }
+        gsl_multifit_nlinear_init(pars->mpopt, &fdf, pars->w);
 
-    /* compute initial cost function */
-    double chisq_init = GSL_POSINF;
+    /* initial cost */
+    double chisq_init = (double)GSL_POSINF;
     gsl_vector *resid = gsl_multifit_nlinear_residual(pars->w);
     gsl_blas_ddot(resid, resid, &chisq_init);
     double chisq0 = chisq_init;
@@ -195,13 +296,13 @@ SEXP C_nls_internal(void *data)
         SET_REAL_ELT(params.ssrtrace, 0, chisq_init);
         double *parptr = REAL(params.partrace);
         for (R_len_t k = 0; k < p; k++)
-            parptr[(niter + 1) * k] = start1[k];
+            parptr[(niter + 1) * k] = gsl_vector_get(pars->mpopt, k);
     }
 
     /* solve the system  */
     int info = GSL_CONTINUE;
     int status = gsl_multifit_nlinear_driver2(niter, xtol, gtol, ftol, verbose ? callback : NULL, verbose ? &params : NULL, &info, &chisq0, &chisq1, pars->w);
-    R_len_t iter = gsl_multifit_nlinear_niter(pars->w);
+    R_len_t iter = (R_len_t)gsl_multifit_nlinear_niter(pars->w);
 
     /* compute covariance and cost at best fit parameters */
     gsl_matrix *J = NULL;
@@ -223,9 +324,9 @@ SEXP C_nls_internal(void *data)
         Rprintf("final ssr = %g\n", chisq1);
         Rprintf("ssr/dof = %g\n", chisq1 / (n - p));
         Rprintf("ssr achieved tolerance = %g\n", chisq0 - chisq1);
-        Rprintf("function evaluations: %d\n", fdf.nevalf);
-        Rprintf("jacobian evaluations: %d\n", fdf.nevaldf);
-        Rprintf("fvv evaluations: %d\n", fdf.nevalfvv);
+        Rprintf("function evaluations: %d\n", (int)fdf.nevalf);
+        Rprintf("jacobian evaluations: %d\n", (int)fdf.nevaldf);
+        Rprintf("fvv evaluations: %d\n", (int)fdf.nevalfvv);
         Rprintf("status = %s\n*******************\n", gsl_strerror(status));
     }
 
@@ -250,12 +351,12 @@ SEXP C_nls_internal(void *data)
     if (status == GSL_SUCCESS || status == GSL_EMAXITER)
     {
         for (R_len_t k = 0; k < p; k++)
-            SET_REAL_ELT(anspar, k, gsl_vector_get(pars->w->x, k));
+            SET_REAL_ELT(anspar, k, gsl_vector_get((pars->w)->x, k));
     }
     else
     {
         for (R_len_t k = 0; k < p; k++)
-            SET_REAL_ELT(anspar, k, REAL_ELT(startvec, k));
+            SET_REAL_ELT(anspar, k, gsl_vector_get(pars->mpopt, k));
     }
     if (!Rf_isNull(parnames))
         Rf_setAttrib(anspar, R_NamesSymbol, parnames);
@@ -346,9 +447,9 @@ SEXP C_nls_internal(void *data)
 
     const char *nms[] = {"f", "J", "fvv", ""};
     SEXP ansneval = PROTECT(Rf_mkNamed(INTSXP, nms));
-    SET_INTEGER_ELT(ansneval, 0, fdf.nevalf);
-    SET_INTEGER_ELT(ansneval, 1, fdf.nevaldf);
-    SET_INTEGER_ELT(ansneval, 2, fdf.nevalfvv);
+    SET_INTEGER_ELT(ansneval, 0, (R_len_t)fdf.nevalf);
+    SET_INTEGER_ELT(ansneval, 1, (R_len_t)fdf.nevaldf);
+    SET_INTEGER_ELT(ansneval, 2, (R_len_t)fdf.nevalfvv);
     SET_VECTOR_ELT(ans, 10, ansneval);
     UNPROTECT(1);
 
@@ -404,11 +505,11 @@ GSL_EBADFUNC if function evaluation failed
 GSL_MAXITER if maxiter exceeded without converging
 GSL_ENOPROG if no accepted step found on first iteration
 */
-int gsl_multifit_nlinear_driver2(const size_t maxiter,
+int gsl_multifit_nlinear_driver2(const R_len_t maxiter,
                                  const double xtol,
                                  const double gtol,
                                  const double ftol,
-                                 void (*callback)(const size_t iter, void *params,
+                                 void (*callback)(const R_len_t iter, void *params,
                                                   const gsl_multifit_nlinear_workspace *w),
                                  void *callback_params,
                                  int *info,
@@ -417,7 +518,7 @@ int gsl_multifit_nlinear_driver2(const size_t maxiter,
                                  gsl_multifit_nlinear_workspace *w)
 {
     int status = GSL_CONTINUE;
-    size_t iter = 0;
+    R_len_t iter = 0;
     gsl_vector *f = NULL;
 
     do
@@ -431,19 +532,19 @@ int gsl_multifit_nlinear_driver2(const size_t maxiter,
         f = gsl_multifit_nlinear_residual(w);
         gsl_blas_ddot(f, f, chisq1);
 
-        if(callback)
+        if (callback)
             ((fdata *)callback_params)->chisq = chisq1[0];
 
         /*
-       * If the solver reports no progress on the first iteration,
-       * then it didn't find a single step to reduce the
-       * cost function and more iterations won't help so return.
-       *
-       * If we get a no progress flag on subsequent iterations,
-       * it means we did find a good step in a previous iteration,
-       * so continue iterating since the solver has now reset
-       * mu to its initial value.
-       */
+         * If the solver reports no progress on the first iteration,
+         * then it didn't find a single step to reduce the
+         * cost function and more iterations won't help so return.
+         *
+         * If we get a no progress flag on subsequent iterations,
+         * it means we did find a good step in a previous iteration,
+         * so continue iterating since the solver has now reset
+         * mu to its initial value.
+         */
         if (status == GSL_EBADFUNC || (status == GSL_ENOPROG && iter == 0))
         {
             *info = status;
@@ -460,10 +561,10 @@ int gsl_multifit_nlinear_driver2(const size_t maxiter,
     } while (status == GSL_CONTINUE && iter < maxiter);
 
     /*
-   * the following error codes mean that the solution has converged
-   * to within machine precision, so record the error code in info
-   * and return success
-   */
+     * the following error codes mean that the solution has converged
+     * to within machine precision, so record the error code in info
+     * and return success
+     */
     if (status == GSL_ETOLF || status == GSL_ETOLX || status == GSL_ETOLG)
     {
         *info = status;
@@ -505,7 +606,8 @@ int gsl_f(const gsl_vector *x, void *params, gsl_vector *f)
     R_len_t n = ((fdata *)params)->n;
     if (TYPEOF(fval) != REALSXP || Rf_length(fval) != n)
     {
-        Rf_warning("Evaluating fn does not return numeric vector of expected length n");
+        if (((fdata *)params)->warn)
+            Rf_warning("Evaluating fn does not return numeric vector of expected length n");
         UNPROTECT(2);
         return GSL_EBADFUNC;
     }
@@ -516,7 +618,7 @@ int gsl_f(const gsl_vector *x, void *params, gsl_vector *f)
     for (R_len_t i = 0; i < n; i++)
     {
         if (R_IsNaN(fvalptr[i]) || !R_finite(fvalptr[i]))
-            gsl_vector_set(f, i, GSL_POSINF);
+            gsl_vector_set(f, i, (double)GSL_POSINF);
         else
             gsl_vector_set(f, i, fvalptr[i] - yptr[i]);
     }
@@ -553,7 +655,8 @@ int gsl_df(const gsl_vector *x, void *params, gsl_matrix *J)
     R_len_t n = ((fdata *)params)->n;
     if (TYPEOF(dfval) != REALSXP || !Rf_isMatrix(dfval) || Rf_ncols(dfval) != p || Rf_nrows(dfval) != n)
     {
-        Rf_warning("Evaluating jac does not return numeric matrix of dimensions n x p");
+        if (((fdata *)params)->warn)
+            Rf_warning("Evaluating jac does not return numeric matrix of dimensions n x p");
         UNPROTECT(2);
         return GSL_EBADFUNC;
     }
@@ -563,7 +666,8 @@ int gsl_df(const gsl_vector *x, void *params, gsl_matrix *J)
         for (R_len_t k = 0; k < p; k++)
             if (R_IsNaN(jacptr[i + n * k]) || !R_finite(jacptr[i + n * k]))
             {
-                Rf_warning("Missing/infinite values not allowed when evaluating jac");
+                if (((fdata *)params)->warn)
+                    Rf_warning("Missing/infinite values not allowed when evaluating jac");
                 UNPROTECT(2);
                 return GSL_EBADFUNC;
             }
@@ -613,7 +717,8 @@ int gsl_fvv(const gsl_vector *x, const gsl_vector *v, void *params, gsl_vector *
     R_len_t n = ((fdata *)params)->n;
     if (TYPEOF(fvvval) != REALSXP || Rf_length(fvvval) != n)
     {
-        Rf_warning("Evaluating fvv does not return numeric vector of expected length n");
+        if (((fdata *)params)->warn)
+            Rf_warning("Evaluating fvv does not return numeric vector of expected length n");
         UNPROTECT(4);
         return GSL_EBADFUNC;
     }
@@ -623,7 +728,8 @@ int gsl_fvv(const gsl_vector *x, const gsl_vector *v, void *params, gsl_vector *
     {
         if (R_IsNaN(fvvvalptr[i]) || !R_finite(fvvvalptr[i]))
         {
-            Rf_warning("Missing/infinite values not allowed when evaluating fvv");
+            if (((fdata *)params)->warn)
+                Rf_warning("Missing/infinite values not allowed when evaluating fvv");
             UNPROTECT(4);
             return GSL_EBADFUNC;
         }
@@ -637,11 +743,11 @@ int gsl_fvv(const gsl_vector *x, const gsl_vector *v, void *params, gsl_vector *
     return GSL_SUCCESS;
 }
 
-void callback(const size_t iter, void *params, const gsl_multifit_nlinear_workspace *w)
+void callback(const R_len_t iter, void *params, const gsl_multifit_nlinear_workspace *w)
 {
     /* update traces */
     double chisq = ((fdata *)params)->chisq;
-    SET_REAL_ELT(((fdata *)params)->ssrtrace, (R_len_t)iter, chisq);
+    SET_REAL_ELT(((fdata *)params)->ssrtrace, iter, chisq);
     R_len_t p = ((fdata *)params)->p;
     R_len_t n = (R_len_t)Rf_nrows(((fdata *)params)->partrace);
     double *parptr = REAL(((fdata *)params)->partrace);
@@ -651,8 +757,135 @@ void callback(const size_t iter, void *params, const gsl_multifit_nlinear_worksp
     /* print trace */
     Rprintf("iter %3d: ssr = %g, par = (", iter, chisq);
     for (R_len_t k = 0; k < p; k++)
-        Rprintf((k < (p -1)) ? "%g, " : "%g)\n", parptr[iter + n * k]);
-
+        Rprintf((k < (p - 1)) ? "%g, " : "%g)\n", parptr[iter + n * k]);
 }
 
+void gsl_multistart_driver(pdata *pars,
+                           mdata *mpars,
+                           gsl_multifit_nlinear_fdf *fdff,
+                           SEXP mssr,
+                           const double xtol,
+                           const double ftol,
+                           const double gtol,
+                           Rboolean verbose)
+{
+    // initialize variables
+    int minfo;
+    R_len_t p = (R_len_t)(pars->mp->size);
+    double kd, l0, l1, kd0, kd1;
+    double rcond, mchisq0 = (double)GSL_POSINF, mchisq1 = (double)GSL_POSINF;
+    trust_state_t *trust_state = (trust_state_t *)(pars->w->state);
 
+    /* sample initial points */
+    for (R_len_t nn = 0; nn < mpars->n; nn++)
+    {
+        if ((mpars->ntix)[nn] == 0)
+        {
+            gsl_qrng_get(pars->q, mpars->qmp);
+            for (R_len_t k = 0; k < p; k++)
+            {
+                l0 = (mpars->startptr)[2 * k];
+                l1 = (mpars->startptr)[2 * k + 1];
+
+                if (l1 > l0)
+                {
+                    kd = gsl_vector_get(pars->diag, k);
+                    kd0 = 1.0 / (1.0 + exp(kd / 2.0));
+                    kd1 = 1.0 / (1.0 + exp(-kd / 2.0));
+                    // gsl_matrix_set(pars->mx, nn, k, l0 + (l1 - l0) * (mpars->qmp)[k]);
+                    (mpars->qmp)[k] = kd0 + (kd1 - kd0) * (mpars->qmp)[k];
+                    gsl_matrix_set(pars->mx, nn, k, (l1 + l0) / 2.0 + (l1 - l0) / kd * log((mpars->qmp)[k] / (1 - (mpars->qmp)[k])));
+                }
+                else
+                {
+                    gsl_matrix_set(pars->mx, nn, k, l0);
+                }
+            }
+        }
+
+        /* concentrate points */
+        gsl_matrix_get_row(pars->mp, pars->mx, nn);
+        if (!Rf_isNull(pars->swts))
+            gsl_multifit_nlinear_winit(pars->mp, pars->wts, fdff, pars->w);
+        else
+            gsl_multifit_nlinear_init(pars->mp, fdff, pars->w);
+        gsl_multifit_nlinear_driver2(mpars->p, xtol, 1e-3, ftol, NULL, NULL, &minfo, &mchisq0, &mchisq1, pars->w);
+        gsl_matrix_set_row(pars->mx, nn, (pars->w)->x);
+        if (mchisq1 < (double)GSL_POSINF)
+            SET_REAL_ELT(mssr, nn, mchisq1);
+        else
+            SET_REAL_ELT(mssr, nn, NA_REAL);
+    }
+    /* reduce sample points */
+    R_orderVector1(mpars->mssr_order, mpars->n, mssr, TRUE, FALSE);
+    for (R_len_t nn = 0; nn < mpars->n; nn++)
+        (mpars->ntix)[(mpars->mssr_order)[nn]] = (nn < (mpars->q)) ? (mpars->ntix)[(mpars->mssr_order)[nn]] + 1 : 0;
+
+    /* local optimization stage */
+    for (R_len_t nn = 0; nn < (mpars->n); nn++)
+    {
+        if ((mpars->ntix)[nn] >= (mpars->s) && !R_IsNA(REAL_ELT(mssr, nn)))
+        {
+            (mpars->ntix)[nn] = 0;
+            if ((mpars->nsp) == 0 || REAL_ELT(mssr, nn) < (1 + (mpars->tol)) * (mpars->mssropt))
+            {
+                gsl_matrix_get_row(pars->mp, pars->mx, nn);
+                if (!Rf_isNull(pars->swts))
+                    gsl_multifit_nlinear_winit(pars->mp, pars->wts, fdff, pars->w);
+                else
+                    gsl_multifit_nlinear_init(pars->mp, fdff, pars->w);
+                mchisq1 = REAL_ELT(mssr, nn);
+                gsl_multifit_nlinear_driver2(mpars->niter, xtol, 1e-3, ftol, NULL, NULL, &minfo, &mchisq0, &mchisq1, pars->w);
+                gsl_multifit_nlinear_rcond(&rcond, pars->w);
+                if (mchisq1 < (mpars->mssropt) && ((!gsl_isnan(rcond) && rcond > gtol) || mchisq1 < 2 * ftol))
+                {
+                    mpars->mssropt = mchisq1;
+                    mpars->nsp += 1;
+                    mpars->nwsp = 0;
+                    gsl_vector_memcpy(pars->mpopt, (pars->w)->x);
+                    gsl_vector_memcpy(pars->diag, trust_state->diag);
+                    if (verbose)
+                    {
+                        Rprintf("mstart ssr* = %g, cond(J) = %g, NSP = %d, NWSP = %d, par = (", mpars->mssropt, 1.0 / rcond, mpars->nsp, mpars->nwsp);
+                        for (R_len_t k = 0; k < p; k++)
+                            Rprintf((k < (p - 1)) ? "%g, " : "%g)\n", gsl_vector_get((pars->w)->x, k));
+                    }
+                }
+                else
+                    mpars->nwsp += 1;
+            }
+            else
+                mpars->nwsp += 1;
+        }
+    }
+}
+
+/* deprecated function */
+// int gsl_evalf(SEXP par, fdata *params, gsl_vector *f)
+// {
+//     /* evaluate function f */
+//     SETCADR(params->f, par);
+//     SEXP fval = PROTECT(Rf_eval(params->f, params->rho));
+
+//     /* function checks */
+//     R_len_t n = params->n;
+//     if (TYPEOF(fval) != REALSXP || Rf_length(fval) != n)
+//     {
+//         UNPROTECT(1);
+//         return GSL_EBADFUNC;
+//     }
+
+//     /* set gsl residuals */
+//     double *fvalptr = REAL(fval);
+//     double *yptr = REAL(params->y);
+//     for (R_len_t i = 0; i < n; i++)
+//     {
+//         if (R_IsNaN(fvalptr[i]) || !R_finite(fvalptr[i]))
+//             gsl_vector_set(f, i, GSL_POSINF);
+//         else
+//             gsl_vector_set(f, i, fvalptr[i] - yptr[i]);
+//     }
+
+//     UNPROTECT(1);
+//     return GSL_SUCCESS;
+// }
